/*
 * Syncdaemon API
 *
 * Authors: Rodrigo Moya <rodrigo.moya@canonical.com>
 *
 * Copyright 2010 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"
#ifdef HAVE_GDBUS
#else
#include <dbus/dbus-glib.h>
#endif
#include "syncdaemon-filesystem-interface.h"

G_DEFINE_TYPE(SyncdaemonFilesystemInterface, syncdaemon_filesystem_interface, SYNCDAEMON_TYPE_INTERFACE)

struct _SyncdaemonFilesystemInterfacePrivate {
	GObject *proxy;
};

static void
syncdaemon_filesystem_interface_finalize (GObject *object)
{
	SyncdaemonFilesystemInterface *interface = SYNCDAEMON_FILESYSTEM_INTERFACE (object);

	if (interface->priv != NULL) {
		g_free (interface->priv);
	}

	G_OBJECT_CLASS (syncdaemon_filesystem_interface_parent_class)->finalize (object);
}

static void
syncdaemon_filesystem_interface_class_init (SyncdaemonFilesystemInterfaceClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	object_class->finalize = syncdaemon_filesystem_interface_finalize;
}

static void
syncdaemon_filesystem_interface_init (SyncdaemonFilesystemInterface *interface)
{
	interface->priv = g_new0 (SyncdaemonFilesystemInterfacePrivate, 1);

	/* Setup DBus proxy */
	interface->priv->proxy = syncdaemon_interface_setup_proxy (SYNCDAEMON_INTERFACE (interface),
								   "com.ubuntuone.Syncdaemon",
								   "/filesystem", "com.ubuntuone.Syncdaemon.FileSystem");
}

/**
 * syncdaemon_filesystem_interface_new:
 */
SyncdaemonFilesystemInterface *
syncdaemon_filesystem_interface_new (SyncdaemonDaemon *daemon)
{
	g_return_val_if_fail (SYNCDAEMON_IS_DAEMON (daemon), NULL);

	return g_object_new (SYNCDAEMON_TYPE_FILESYSTEM_INTERFACE, "daemon", daemon, NULL);
}

static void
got_metadata_cb (DBusGProxy *proxy, DBusGProxyCall *call_id, gpointer user_data)
{
	GError *error = NULL;
	GHashTable *metadata;
	SyncdaemonFilesystemInterface *interface = SYNCDAEMON_FILESYSTEM_INTERFACE (user_data);

	if (dbus_g_proxy_end_call (proxy, call_id, &error,
				   dbus_g_type_get_map ("GHashTable",
							G_TYPE_STRING,
							G_TYPE_STRING), &metadata,
				   G_TYPE_INVALID)) {
		SyncdaemonDaemon *daemon = NULL;

		g_object_get (G_OBJECT (interface), "daemon", &daemon, NULL);
		if (daemon != NULL)
			g_signal_emit_by_name (daemon, "got_metadata",
					       g_hash_table_lookup (metadata, "path"),
					       metadata);

		g_hash_table_destroy (metadata);
	}
}

/**
 * syncdaemon_filesystem_interface_get_metadata:
 */
void
syncdaemon_filesystem_interface_get_metadata (SyncdaemonFilesystemInterface *interface,
					      const gchar *path)
{
	g_return_if_fail (SYNCDAEMON_IS_FILESYSTEM_INTERFACE (interface));

	dbus_g_proxy_begin_call (DBUS_G_PROXY (interface->priv->proxy), "get_metadata",
				 got_metadata_cb, interface, NULL,
				 G_TYPE_STRING, path,
				 G_TYPE_INVALID);
}
