/*
 * Syncdaemon API
 *
 * Authors: Rodrigo Moya <rodrigo.moya@canonical.com>
 *
 * Copyright 2010 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"
#include "syncdaemon-publicfiles-interface.h"
#include "utils.h"

G_DEFINE_TYPE(SyncdaemonPublicfilesInterface, syncdaemon_publicfiles_interface, SYNCDAEMON_TYPE_INTERFACE)

struct _SyncdaemonPublicfilesInterfacePrivate {
	GObject *proxy;
};

static void
syncdaemon_publicfiles_interface_finalize (GObject *object)
{
	SyncdaemonPublicfilesInterface *interface = SYNCDAEMON_PUBLICFILES_INTERFACE (object);

	if (interface->priv != NULL) {
		g_free (interface->priv);
	}

	G_OBJECT_CLASS (syncdaemon_publicfiles_interface_parent_class)->finalize (object);
}

static void
syncdaemon_publicfiles_interface_class_init (SyncdaemonPublicfilesInterfaceClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	object_class->finalize = syncdaemon_publicfiles_interface_finalize;
}

static void
syncdaemon_publicfiles_interface_init (SyncdaemonPublicfilesInterface *interface)
{
	interface->priv = g_new0 (SyncdaemonPublicfilesInterfacePrivate, 1);

	/* Setup DBus proxy */
	interface->priv->proxy = syncdaemon_interface_setup_proxy (SYNCDAEMON_INTERFACE (interface),
								   "com.ubuntuone.Syncdaemon",
								   "/publicfiles", "com.ubuntuone.Syncdaemon.PublicFiles");
	if (interface->priv->proxy != NULL) {
		/* FIXME: connect to /publicfiles signals */
	}
}

/**
 * syncdaemon_publicfiles_interface_new:
 */
SyncdaemonPublicfilesInterface *
syncdaemon_publicfiles_interface_new (SyncdaemonDaemon *daemon)
{
	g_return_val_if_fail (SYNCDAEMON_IS_DAEMON (daemon), NULL);

	return g_object_new (SYNCDAEMON_TYPE_PUBLICFILES_INTERFACE, "daemon", daemon, NULL);
}

/**
 * syncdaemon_publicfiles_interface_change_public_access:
 */
void
syncdaemon_publicfiles_interface_change_public_access (SyncdaemonPublicfilesInterface *interface,
						       const gchar *share_id,
						       const gchar *node_id,
						       gboolean is_public)
{
	g_return_if_fail (SYNCDAEMON_IS_PUBLICFILES_INTERFACE (interface));
	g_return_if_fail (share_id != NULL);
	g_return_if_fail (node_id != NULL);

	dbus_g_proxy_begin_call (DBUS_G_PROXY (interface->priv->proxy), "change_public_access",
				 no_output_dbus_call_ended_cb, interface, NULL,
				 G_TYPE_STRING, share_id,
				 G_TYPE_STRING, node_id,
				 G_TYPE_BOOLEAN, is_public,
				 G_TYPE_INVALID);
}

/**
 * syncdaemon_publicfiles_interface_get_public_files:
 */
void
syncdaemon_publicfiles_interface_get_public_files (SyncdaemonPublicfilesInterface *interface)
{
	g_return_if_fail (SYNCDAEMON_IS_PUBLICFILES_INTERFACE (interface));

	dbus_g_proxy_begin_call (DBUS_G_PROXY (interface->priv->proxy), "get_public_files",
				 no_output_dbus_call_ended_cb, interface, NULL,
				 G_TYPE_INVALID);
}
